# refs: System.Web.Extensions

import System
import System.Text.RegularExpressions
import System.Web.Script.Serialization
import AlbumArtDownloader.Scripts
import util

class Discogs(AlbumArtDownloader.Scripts.IScript):
	def constructor():
		// Discogs requires TLS 1.1 or greater
		Tls12 = Enum.ToObject(typeof(System.Net.SecurityProtocolType), 3072);
		Tls11 = Enum.ToObject(typeof(System.Net.SecurityProtocolType), 768);

		System.Net.ServicePointManager.SecurityProtocol = Tls12 | Tls11 | System.Net.SecurityProtocolType.Tls
	Name as string:
		get: return "Discogs"
	Author as string:
		get: return "Alex Vallat"
	Version as string:
		get: return "0.19"
	def Search(artist as string, album as string, results as IScriptResults):
		//artist = StripCharacters("&.'\";:?!", artist)
		//album = StripCharacters("&.'\";:?!", album)

		json = JavaScriptSerializer()

		resultsInfoJson = GetDiscogsPage("https://www.discogs.com/search/ac?searchType=release&q=" + EncodeUrl("\"${artist}\" \"${album}\""))
		resultsInfo = json.Deserialize[of (Result)](resultsInfoJson)
		
		results.EstimatedCount = resultsInfo.Length;
		
		for result in resultsInfo:
			// Get the release info from api
			title = result.GetString(result.artist) + " - " + result.GetString(result.title);
			url = result.GetString(result.uri);
			//id = url.Substring(url.LastIndexOf('/'));

			releasePage = GetDiscogsPage("http://www.discogs.com" + url)
			releasePageImagesMatch = Regex("data-images='(?<json>[^']+)'", RegexOptions.IgnoreCase | RegexOptions.Singleline).Match(releasePage)
			if releasePageImagesMatch.Success:
				imagesJson = releasePageImagesMatch.Groups["json"].Value
				images = json.Deserialize[of (Image)](imagesJson)
			
				results.EstimatedCount += images.Length - 1
				for image in images:
					results.Add(GetPageStream(image.thumb, null, true), title, "http://www.discogs.com" + url, image.width, image.height, image.full, CoverType.Unknown)

	def RetrieveFullSizeImage(fullSizeCallbackParameter):
		return GetPageStream(fullSizeCallbackParameter, null, true);

	def GetDiscogsPage(url):
		request = System.Net.HttpWebRequest.Create(url) as System.Net.HttpWebRequest

		request.Accept = "*/*"
		request.UserAgent = "curl/7.38.0"
			
		response = request.GetResponse()
		stream = response.GetResponseStream();
		try:
			return GetPage(stream)
		ensure:
			stream.Close()
	
	class Result:
		public artist as object
		public title as object
		public uri as object

		def GetString(value):
			result = value as string;
			if result is null:
				result = (value as (object))[0] as string;
			return result;


	class Image:
		public thumb as String
		public full as string
		public width as int
		public height as int
